#!/bin/sh

#                                     ╭───────────────╮
#                                     │ env functions │
#                                     ╰───────────────╯
# ───────────────────────────────────< ANSI color codes >───────────────────────────────────
RED='\033[0;31m'
CYAN='\033[0;36m'
YELLOW='\033[0;33m'
LIGHT_GREEN='\033[0;92m'
BOLD='\033[1m'
NC='\033[0m' # No Color

echo_error() {
  printf "${BOLD}${RED}ERROR: ${NC}${RED}%s${NC}\n" "$1" >&2
}

echo_info() {
  printf "${BOLD}${CYAN}INFO: ${NC}${CYAN}%s${NC}\n" "$1"
}

echo_warning() {
  printf "${BOLD}${YELLOW}WARNING: ${NC}${YELLOW}%s${NC}\n" "$1"
}

echo_note() {
  printf "${BOLD}${LIGHT_GREEN}NOTE: ${NC}${LIGHT_GREEN}%s${NC}\n" "$1"
}

command_exists() {
  command -v "$@" >/dev/null 2>&1
}

# ─────────────< Check if the user is root and set sudo variable if necessary >─────────────
check_root() {
  if [ "$(id -u)" -ne 0 ]; then
    if command_exists sudo; then
      echo_info "User is not root. Using sudo for privileged operations."
      _sudo="sudo"
    else
      echo_error "No sudo found and you're not root! Can't install packages."
      return 1
    fi
  else
    echo_info "Root access confirmed."
    _sudo=""
  fi
}

# ──────────────────────< Check if the given command exists silently >──────────────────────
command_exists() {
  command -v "$@" >/dev/null 2>&1
}

#                          ╭────────────────────────────────────╮
#                          │ insert your scripts/functions here │
#                          ╰────────────────────────────────────╯

# Function to initialize or update the Docker daemon configuration
dnetwork_init() {
  if ! command_exists docker; then
    echo_error "No docker was found! Cannot continue!"
    return 1
  fi

  CONFIG_FILE="/etc/docker/daemon.json"

  echo_info "Checking Docker daemon configuration..."

  # Check if the configuration file exists
  if [ ! -f "$CONFIG_FILE" ]; then
    echo_warning "Docker daemon configuration file not found. Creating it."
    ${_sudo} mkdir -p /etc/docker
    ${_sudo} tee "$CONFIG_FILE" >/dev/null <<EOF
{
  "default-address-pools": [
    {
      "base": "10.0.0.0/8",
      "size": 24
    }
  ]
}
EOF
    echo_note "Configuration file created with default network."
  else
    # Check if the "default-address-pools" entry exists
    if grep -q '"default-address-pools"' "$CONFIG_FILE"; then
      echo_note "Docker daemon configuration already contains 'default-address-pools'. No changes made."
    else
      echo_warning "Adding 'default-address-pools' to existing configuration."
      ${_sudo} jq '. + {"default-address-pools": [{"base": "10.0.0.0/8", "size": 24}]}' "$CONFIG_FILE" | ${_sudo} tee "$CONFIG_FILE.tmp" >/dev/null
      ${_sudo} mv "$CONFIG_FILE.tmp" "$CONFIG_FILE"
      echo_note "Default address pool added to configuration."
    fi
  fi

  echo_info "Restarting Docker to apply changes..."
  ${_sudo} systemctl restart docker && echo_note "Docker restarted successfully." || echo_error "Failed to restart Docker."
}

# ───────────────────────────────< main function to execute >───────────────────────────────
main() {
  if check_root; then
    dnetwork_init
  else
    echo_error "Something went terribly wrong!"
  fi
}

main
